﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace ThinkGeoRefreshIssues
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(0, 100, 100, 0);
            wpfMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.White);

            InMemoryFeatureLayer inMemoryLayer = new InMemoryFeatureLayer();
            inMemoryLayer.InternalFeatures.Add("Polygon", new Feature("POLYGON((10 60,40 70,30 85, 10 60))", "Polygon"));
            inMemoryLayer.InternalFeatures.Add("Rectangle", new Feature(new RectangleShape(65, 30, 95, 15).GetWellKnownText(), "Rectangle"));

            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(100, GeoColor.StandardColors.RoyalBlue);
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay inmemoryOverlay = new LayerOverlay();
            inmemoryOverlay.TileType = TileType.MultipleTile;
            inmemoryOverlay.Layers.Add("InMemoryFeatureLayer", inMemoryLayer);
            wpfMap1.Overlays.Add("InmemoryOverlay", inmemoryOverlay);

            wpfMap1.Refresh();
        }

        private void btnEditAFeature_Click(object sender, RoutedEventArgs e)
        {
            InMemoryFeatureLayer inMemoryLayer = (InMemoryFeatureLayer)wpfMap1.FindFeatureLayer("InMemoryFeatureLayer");
            LayerOverlay overlay = (LayerOverlay)wpfMap1.Overlays["InmemoryOverlay"];

            Feature polygonFeature = inMemoryLayer.InternalFeatures["Polygon"];
            Feature rectangleFeature = inMemoryLayer.InternalFeatures["Rectangle"];

            #region refresh calls cancel each other

            inMemoryLayer.FeatureIdsToExclude.Add("Polygon");
            overlay.Refresh(polygonFeature.GetBoundingBox());

            inMemoryLayer.FeatureIdsToExclude.Add("Rectangle");
            overlay.Refresh(rectangleFeature.GetBoundingBox());

            #endregion
        }

        private void btnEditAFeature_Click2(object sender, RoutedEventArgs e)
        {
            InMemoryFeatureLayer inMemoryLayer = (InMemoryFeatureLayer)wpfMap1.FindFeatureLayer("InMemoryFeatureLayer");
            LayerOverlay overlay = (LayerOverlay)wpfMap1.Overlays["InmemoryOverlay"];

            Feature polygonFeature = inMemoryLayer.InternalFeatures["Polygon"];
            Feature rectangleFeature = inMemoryLayer.InternalFeatures["Rectangle"];

            #region Refresh(Enumerable<RectangleShape>) seems to be broken

            inMemoryLayer.FeatureIdsToExclude.Add("Polygon");
            inMemoryLayer.FeatureIdsToExclude.Add("Rectangle");
            List<RectangleShape> shapes = new List<RectangleShape>()
            {
                polygonFeature.GetBoundingBox(),
                rectangleFeature.GetBoundingBox()
            };

            overlay.Refresh(shapes);

            #endregion
        }

        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            InMemoryFeatureLayer inMemoryLayer = (InMemoryFeatureLayer)wpfMap1.FindFeatureLayer("InMemoryFeatureLayer");
            inMemoryLayer.FeatureIdsToExclude.Clear();

            wpfMap1.Refresh();
        }

        private void ButtonBase_OnClick2(object sender, RoutedEventArgs e)
        {
            InMemoryFeatureLayer inMemoryLayer = (InMemoryFeatureLayer)wpfMap1.FindFeatureLayer("InMemoryFeatureLayer");
            LayerOverlay overlay = (LayerOverlay)wpfMap1.Overlays["InmemoryOverlay"];

            inMemoryLayer.FeatureIdsToExclude.Add("Polygon");
            inMemoryLayer.FeatureIdsToExclude.Add("Rectangle");

            Feature polygonFeature = inMemoryLayer.InternalFeatures["Polygon"];
            Feature rectangleFeature = inMemoryLayer.InternalFeatures["Rectangle"];

            RectangleShape polygonBounds = polygonFeature.GetBoundingBox();
            RectangleShape rectangleBounds = rectangleFeature.GetBoundingBox();

            overlay.Refresh(polygonBounds.Union(rectangleBounds).GetBoundingBox());
        }
    }
}
